@@ -47,8 +47,7 @@ void confwrite_list(FILE *out, char *prefix, int val, struct keyword_def *k)
 
 void confwrite_int(FILE *out,
 		   char   *side,
-		   unsigned int context,
-		   unsigned int keying_context,
+		   int     context,
 		   knf     options,
 		   int_set options_set,
 		   ksf     strings)
@@ -57,8 +56,7 @@ void confwrite_int(FILE *out,
 
     for(k=ipsec_conf_keywords_v2; k->keyname!=NULL; k++) {
 	
-	if((k->validity & KV_CONTEXT_MASK) != context) continue;
-	if(keying_context != 0 && (k->validity & keying_context)==0) continue;
+	if((k->validity & context)!=context) continue;
 
 	/* do not output aliases */
 	if(k->validity & kv_alias) continue;
@@ -68,14 +66,13 @@ void confwrite_int(FILE *out,
 	if(k->validity & kv_processed) continue;
 
 #if 0
-	printf("#side: %s  %s validity: %08x & %08x=%08x vs %08x\n", side,
-	       k->keyname, k->validity, KV_CONTEXT_MASK, k->validity&KV_CONTEXT_MASK, context);
+	printf("side: %s  %s validity: %08x & %08x=%08x\n", side,
+	       k->keyname, k->validity, context, k->validity&context);
 #endif
 	
 	switch(k->type) {
 	case kt_string:
 	case kt_appendstring:
-	case kt_appendlist:
 	case kt_filename:
 	case kt_dirname:
 	case kt_rsakey:
@@ -148,9 +145,6 @@ void confwrite_int(FILE *out,
 
 	case kt_number:
 	    break;
-
-	case kt_comment:
-	    continue;
 	}
 
 	if(options_set[k->field]) {
@@ -162,16 +156,13 @@ void confwrite_int(FILE *out,
 void confwrite_str(FILE *out,
 		   char   *side,
 		   int     context,
-		   int     keying_context,
 		   ksf     strings,
 		   str_set strings_set)
 {
     struct keyword_def *k;
 
     for(k=ipsec_conf_keywords_v2; k->keyname!=NULL; k++) {
-
-	if((k->validity & KV_CONTEXT_MASK)!=context) continue;
-	if(keying_context != 0 && (k->validity & keying_context)==0) continue;
+	if((k->validity & context)==0) continue;
 
 	/* do not output aliases */
 	if(k->validity & kv_alias) continue;
@@ -181,12 +172,6 @@ void confwrite_str(FILE *out,
 	if(k->validity & kv_processed) continue;
 	
 	switch(k->type) {
-	case kt_appendlist:
-	    if(strings_set[k->field]) {
-		fprintf(out, "\t%s%s={%s}\n",side, k->keyname, strings[k->field]);
-	    }
-	    continue;	    
-
 	case kt_string:
 	case kt_appendstring:
 	case kt_filename:
@@ -216,20 +201,10 @@ void confwrite_str(FILE *out,
 	case kt_percent:
 	case kt_number:
 	    continue;
-
-	case kt_comment:
-	    continue;
 	}
 
 	if(strings_set[k->field]) {
-	    char *quote="";
-
-	    if(strchr(strings[k->field],' ')) quote="\"";
-	    
-	    fprintf(out, "\t%s%s=%s%s%s\n",side, k->keyname
-		    , quote
-		    , strings[k->field]
-		    , quote);
+	    fprintf(out, "\t%s%s=%s\n",side, k->keyname, strings[k->field]);
 	}
     }	
 }    
@@ -278,12 +253,6 @@ void confwrite_side(FILE *out,
 	fprintf(out, "\t%s=%%group\n",side);   
 	break;
 
-    case KH_IPHOSTNAME:
-	if(end->strings_set[KSCF_IP]) {
-	    fprintf(out, "\t%s=%s\n",side, end->strings[KSCF_IP]);
-	}
-	break;
-
     case KH_IPADDR:
 	addrtot(&end->addr, 0, databuf, ADDRTOT_BUF);
 	fprintf(out, "\t%s=%s\n", side, databuf);
@@ -351,17 +320,11 @@ void confwrite_side(FILE *out,
 	fprintf(out, "\t%scert=%s\n", side, end->cert);
     }
 
-    if(!isanyaddr(&end->sourceip)) {
-	addrtot(&end->sourceip, 0, databuf, ADDRTOT_BUF);
-	fprintf(out, "\t%ssourceip=%s\n", side, databuf);
-    }
-
     confwrite_int(out, side,
-		  kv_conn|kv_leftright,
-		  keyingtype,
+		  keyingtype|kv_conn|kv_leftright,
 		  end->options, end->options_set, end->strings);
-    confwrite_str(out, side, kv_conn|kv_leftright,
-		  keyingtype,
+    confwrite_str(out, side,
+		  keyingtype|kv_conn|kv_leftright,
 		  end->strings, end->strings_set);
 
 }
@@ -395,17 +358,11 @@ void confwrite_conn(FILE *out,
     }
     confwrite_side(out, conn, &conn->left,  "left");
     confwrite_side(out, conn, &conn->right, "right");
-    confwrite_int(out, "", kv_conn,
-		  keyingtype,
+    confwrite_int(out, "", keyingtype|kv_conn,
 		  conn->options, conn->options_set, conn->strings);
-    confwrite_str(out, "", kv_conn,
-		  keyingtype,
+    confwrite_str(out, "", keyingtype|kv_conn,
 		  conn->strings, conn->strings_set);
 
-    if(conn->connalias) {
-	fprintf(out, "\tconnalias=\"%s\"\n", conn->connalias);
-    }
-
     if(conn->manualkey) {
 	fprintf(out, "\tmanual=add\n");
     } else {
@@ -460,9 +417,9 @@ void confwrite_conn(FILE *out,
 	    }
 	    
 	    if(conn->policy & POLICY_DONT_REKEY) {
-		fprintf(out, "\trekey=no  #duplicate?\n");
+		fprintf(out, "\tnorekey=yes\n");
 	    } else {
-		fprintf(out, "\trekey=yes\n");
+		fprintf(out, "\tnorekey=no\n");
 	    }
 	    
 	    auth_policy=(conn->policy & POLICY_ID_AUTH_MASK);
@@ -545,11 +502,9 @@ void confwrite(struct starter_config *cfg, FILE *out)
 
 	/* output config setup section */
 	fprintf(out, "config setup\n");
-	confwrite_int(out, "", 
-		      kv_config, 0,
+	confwrite_int(out, "", kv_config,
 		      cfg->setup.options, cfg->setup.options_set, cfg->setup.strings);
-	confwrite_str(out, "", 
-		      kv_config, 0,
+	confwrite_str(out, "", kv_config,
 		      cfg->setup.strings, cfg->setup.strings_set);
 
 	fprintf(out, "\n\n");
