@@ -19,6 +19,7 @@
 #include <limits.h>
 #include <assert.h>
 #include <sys/queue.h>
+#include <stdio.h>
 
 #include "ipsecconf/parser.h"
 #include "ipsecconf/confread.h"
@@ -339,12 +340,14 @@ struct oe_conn oe_private = {
 	.oe_ct = OE_PRIVATE,
 	.oe_cn = "private",
 	.oe_sc = {
+		/* RSASIG+ENCRYPT+TUNNEL+PFS+DONTREKEY+OPPORTUNISTIC+GROUP
+		 * +GROUTED+failureDROP+lKOD+rKOD
+		 */
+
 		.policy = POLICY_RSASIG|POLICY_ENCRYPT|POLICY_TUNNEL|POLICY_PFS|
-		POLICY_OPPO|POLICY_GROUP|POLICY_GROUTED|
+		POLICY_DONT_REKEY|POLICY_OPPO|POLICY_GROUP|POLICY_GROUTED|
 		POLICY_FAIL_DROP,
-		
-		.options[KBF_REKEY]=FALSE,    /* really want REKEY if used */
-		.options_set[KBF_REKEY]=TRUE,
+
 
 		.desired_state = STARTUP_ROUTE,
 		
@@ -479,7 +482,7 @@ void add_any_oeconns(struct starter_config *cfg,
 	int i;
 
 	for(i=0;i<OE_MAX;i++) found_conns[i]=FALSE;
-	
+
 	/* look for the conn. */
 	for(sconn = cfgp->sections.tqh_first; sconn != NULL; sconn = sconn->link.tqe_next)
 		
