@@ -11,6 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
+ * RCSID $Id: constants.c,v 1.14 2005/09/27 02:19:05 mcr Exp $
  */
 
 /*
@@ -33,22 +34,14 @@
 
 /* version */
 
-static const char *const version_name_1[] = {
-	"ISAKMP Version 1.0 (rfc2407)",
+static const char *const version_name[] = {
+	"ISAKMP Version 1.0",
 };
-static const char *const version_name_2[] = {
-	"IKEv2 version 2.0 (rfc4306)",
-};
-
-enum_names version_names_1 =
-    { ISAKMP_MAJOR_VERSION<<ISA_MAJ_SHIFT | ISAKMP_MINOR_VERSION,
-	ISAKMP_MAJOR_VERSION<<ISA_MAJ_SHIFT | ISAKMP_MINOR_VERSION,
-	version_name_1, NULL };
 
 enum_names version_names =
-    { IKEv2_MAJOR_VERSION<<ISA_MAJ_SHIFT | IKEv2_MINOR_VERSION,
-	IKEv2_MAJOR_VERSION<<ISA_MAJ_SHIFT | IKEv2_MINOR_VERSION,
-	version_name_2, &version_names_1 };
+    { ISAKMP_MAJOR_VERSION<<ISA_MAJ_SHIFT | ISAKMP_MINOR_VERSION,
+	ISAKMP_MAJOR_VERSION<<ISA_MAJ_SHIFT | ISAKMP_MINOR_VERSION,
+	version_name, NULL };
 
 /* Domain of Interpretation */
 
@@ -141,39 +134,12 @@ const char *const payload_name[] = {
 	NULL
     };
 
-const char *const payload_names_ikev2[] = {
-    "ISAKMP_NEXT_v2SA",            /* 33 */
-    "ISAKMP_NEXT_v2KE",
-    "ISAKMP_NEXT_v2IDi",
-    "ISAKMP_NEXT_v2IDr",
-    "ISAKMP_NEXT_v2CERT",  
-    "ISAKMP_NEXT_v2CERTREQ",
-    "ISAKMP_NEXT_v2AUTH",
-    "ISAKMP_NEXT_v2Ni",
-    "ISAKMP_NEXT_v2N",
-    "ISAKMP_NEXT_v2D",
-    "ISAKMP_NEXT_v2V",
-    "ISAKMP_NEXT_v2TSi",
-    "ISAKMP_NEXT_v2TSr",
-    "ISAKMP_NEXT_v2E",
-    "ISAKMP_NEXT_v2CP",
-    "ISAKMP_NEXT_v2EAP",
-    NULL
-};
-
-const char *const payload_name_nat_d[] = {
-    "ISAKMP_NEXT_NAT-D",
-    "ISAKMP_NEXT_NAT-OA",
-    NULL
-};
-
+const char *const payload_name_nat_d[] = { "ISAKMP_NEXT_NAT-D",
+	"ISAKMP_NEXT_NAT-OA", NULL };
 static enum_names payload_names_nat_d =
-{ ISAKMP_NEXT_NATD_DRAFTS, ISAKMP_NEXT_NATOA_DRAFTS, payload_name_nat_d, NULL };
-static enum_names payload_names_ikev2_d =
-{ ISAKMP_NEXT_v2SA, ISAKMP_NEXT_v2EAP, payload_names_ikev2, &payload_names_nat_d };
-
+	{ ISAKMP_NEXT_NATD_DRAFTS, ISAKMP_NEXT_NATOA_DRAFTS, payload_name_nat_d, NULL };
 enum_names payload_names =
-{ ISAKMP_NEXT_NONE, ISAKMP_NEXT_NATOA_RFC, payload_name, &payload_names_ikev2_d };
+    { ISAKMP_NEXT_NONE, ISAKMP_NEXT_NATOA_RFC, payload_name, &payload_names_nat_d };
 
 
 /* Exchange types (note: two discontinuous ranges) */
@@ -191,26 +157,19 @@ static const char *const exchange_name[] = {
 static const char *const exchange_name2[] = {
 	"ISAKMP_XCHG_QUICK",
 	"ISAKMP_XCHG_NGRP",
-	"ISAKMP_v2_SA_INIT",
-	"ISAKMP_v2_AUTH",
-	"ISAKMP_v2_CHILD_SA",
-	"ISAKMP_v2_INFORMATIONAL",
+	"ISAKMP_XCHG_ACK_INFO",
     };
 
 static enum_names exchange_desc2 =
-    { ISAKMP_XCHG_QUICK, ISAKMP_v2_INFORMATIONAL, exchange_name2, NULL };
+    { ISAKMP_XCHG_QUICK, ISAKMP_XCHG_ACK_INFO, exchange_name2, NULL };
 
 enum_names exchange_names =
     { ISAKMP_XCHG_NONE, ISAKMP_XCHG_MODE_CFG, exchange_name, &exchange_desc2 };
 /* Flag BITS */
 const char *const flag_bit_names[] = {
-    "ISAKMP_FLAG_ENCRYPTION",         /* bit 0 */
-    "ISAKMP_FLAG_COMMIT",             /* bit 1 */
-    "bit 2",                          /* bit 2 */
-    "ISAKMP_FLAG_INIT",               /* bit 3 */
-    "ISAKMP_FLAG_VERSION",            /* bit 4 */
-    "ISAKMP_FLAG_RESPONSE",           /* bit 5 */
-    NULL
+	"ISAKMP_FLAG_ENCRYPTION",
+	"ISAKMP_FLAG_COMMIT",
+	NULL
     };
 
 /* Situation BITS definition for IPsec DOI */
@@ -293,14 +252,15 @@ enum_names esp_transformid_names =
 /* IPCOMP transform values */
 
 static const char *const ipcomp_transform_name[] = {
+    "IPCOMP_NONE",
 	"IPCOMP_OUI",
-	"IPCOMP_DEFLAT",
+	"IPCOMP_DEFLATE",
 	"IPCOMP_LZS",
 	"IPCOMP_V42BIS",
     };
 
 enum_names ipcomp_transformid_names =
-    { IPCOMP_OUI, IPCOMP_V42BIS, ipcomp_transform_name, NULL };
+    { IPCOMP_NONE, IPCOMP_V42BIS, ipcomp_transform_name, NULL };
 
 /* Identification type values */
 
@@ -421,7 +381,6 @@ enum_names *oakley_attr_val_descs[] = {
 	NULL,			/* OAKLEY_FIELD_SIZE */
 	NULL,			/* OAKLEY_GROUP_ORDER */
     };
-const unsigned int oakley_attr_val_descs_size = elemsof(oakley_attr_val_descs);
 
 /* IPsec DOI attributes (RFC 2407 "IPsec DOI" section 4.5) */
 
@@ -470,7 +429,6 @@ enum_names *ipsec_attr_val_descs[] = {
 	NULL,			/* COMPRESS_DICT_SIZE */
 	NULL,			/* COMPRESS_PRIVATE_ALG */
     };
-const unsigned int ipsec_attr_val_descs_size=elemsof(ipsec_attr_val_descs);
 
 /* SA Lifetime Type attribute */
 
@@ -699,16 +657,6 @@ enum_names oakley_auth_names =
     { XAUTHInitPreShared, XAUTHRespRSARevisedEncryption
 	, oakley_auth_name3, &oakley_auth_names2 };
 
-/* ikev2 auth methods */
-static const char *const ikev2_auth_strings[]={
-	"v2_AUTH_RSA",
-	"v2_AUTH_SHARED",
-	"v2_AUTH_DSA",
-};
-enum_names ikev2_auth_names =
-{ v2_AUTH_RSA, v2_AUTH_DSA, ikev2_auth_strings, NULL};
-
-
 /* Oakley Group Description attribute */
 
 static const char *const oakley_group_name[] = {
@@ -828,89 +776,6 @@ const char *const attr_msg_type_name[] = {
 enum_names attr_msg_type_names =
     { 0 , ISAKMP_CFG_ACK, attr_msg_type_name , NULL };
 
-/*
- * IKEv2 CRITICAL BYTE "enum"
- */
-const char *const critical_name[]=    { "Payload-Critical", NULL };
-const char *const no_critical_name[]= { "Payload-Non-Critical", NULL };
-enum_names no_crit_names ={ ISAKMP_PAYLOAD_NONCRITICAL,
-			    ISAKMP_PAYLOAD_NONCRITICAL,
-			    no_critical_name, NULL};
-enum_names critical_names = 
-{ ISAKMP_PAYLOAD_CRITICAL, ISAKMP_PAYLOAD_CRITICAL,
-  critical_name, &no_crit_names};
-
-/* Transform-type Encryption */
-const char *const trans_type_encr_name[]={
-    "des-iv64",
-    "des",
-    "3des",
-    "rc5",
-    "idea",
-    "cast",
-    "blowfish",
-    "3idea",
-    "des-iv32",
-    "res10",
-    "null",
-    "aes-cbc",
-    "aes-ctr",
-};
-enum_names trans_type_encr_names =
-{ IKEv2_ENCR_DES_IV64, IKEv2_ENCR_AES_CTR, trans_type_encr_name, NULL};
-
-/* Transform-type PRF */
-const char *const trans_type_prf_name[]={
-    "prf-hmac-md5",
-    "prf-hmac-sha1",
-    "prf-hmac-tiger",
-    "prf-hmac-aes128-xcbc",
-};
-enum_names trans_type_prf_names =
-{ IKEv2_PRF_HMAC_MD5, IKEv2_PRF_AES128_XCBC, trans_type_prf_name, NULL};
-
-/* Transform-type Integrity */
-const char *const trans_type_integ_name[]={
-    "auth-hmac-md5-96",
-    "auth-hmac-sha1-96",
-    "auth-des-mac",
-    "auth-kpdk-md5",
-    "auth-aes-xcbc-96",
-};
-enum_names trans_type_integ_names =
-{ IKEv2_AUTH_HMAC_MD5_96, IKEv2_AUTH_AES_XCBC_96, trans_type_integ_name, NULL};
-
-/* Transform-type Integrity */
-const char *const trans_type_esn_name[]={
-    "esn-disabled",
-    "esn-enabled",
-};
-enum_names trans_type_esn_names =
-{ IKEv2_ESN_DISABLED, IKEv2_ESN_ENABLED, trans_type_esn_name, NULL};
-
-/* Transform Type */
-const char *const trans_type_name[]={
-    "trans-type-encr",
-    "trans-type-prf",
-    "trans-type-integ",
-    "trans-type-dh",
-    "trans-type-esn"
-};
-enum_names trans_type_names =
-{ IKEv2_TRANS_TYPE_ENCR, IKEv2_TRANS_TYPE_ESN, trans_type_name, NULL};
-
-/* for each IKEv2 transform attribute,which enum_names describes its values? */
-enum_names *ikev2_transid_val_descs[] = {
-    NULL,
-    &trans_type_encr_names, /* 1 */
-    &trans_type_prf_names,  /* 2 */
-    &trans_type_integ_names, /* 3 */
-    &oakley_group_names,    /* 4 */
-    &trans_type_esn_names,  /* 5 */
-};
-const unsigned int ikev2_transid_val_descs_size = elemsof(ikev2_transid_val_descs);
-    
-
 /* socket address family info */
 
 static const char *const af_inet_name[] = {
@@ -1044,11 +909,10 @@ static const char *const ppk_name[] = {
   "PPK_DSS",
   "PPK_RSA",
   "PPK_PIN",
-  "PPK_XAUTH",
   NULL
 };
 
-enum_names ppk_names = { PPK_PSK, PPK_XAUTH, ppk_name, NULL };
+enum_names ppk_names = { PPK_PSK, PPK_PIN, ppk_name, NULL };
 
 /*
  * NAT-Traversal defines for nat_traveral type from nat_traversal.h
@@ -1057,10 +921,9 @@ enum_names ppk_names = { PPK_PSK, PPK_XAUTH, ppk_name, NULL };
 const char *const natt_type_bitnames[] = {
   "draft-ietf-ipsec-nat-t-ike-00/01",    /* 0 */
   "draft-ietf-ipsec-nat-t-ike-02/03",
-  "draft-ietf-ipsec-nat-t-ike-05",
-  "draft-ietf-ipsec-nat-t-ike (MacOS X)", 
-  "RFC 3947 (NAT-Traversal)",		/* 4 */
-  "4",   "5",   "6",   "7", 
+  "draft-ietf-ipsec-nat-t-ike (OS X)",
+  "RFC 3947 (NAT-Traversal)",
+  "4",   "5",   "6",   "7",              /* 4 */ 
   "8",   "9",   "10",  "11",
   "12",  "13",  "14",  "15",
   "16",  "17",  "18",  "19", 
@@ -1084,7 +947,6 @@ struct keyword_enum_value kw_host_values[]={
     { "%opportunisticgroup", KH_OPPOGROUP },
     { "%oppogroup",     KH_OPPOGROUP },  
     { "%group",         KH_GROUP },
-    { "%hostname",      KH_IPHOSTNAME },  /* makes no sense on input */
 };
 
 struct keyword_enum_values kw_host_list=
