@@ -41,7 +41,8 @@ __FBSDID("$FreeBSD: src/sys/opencrypto/criov.c,v 1.3 2005/01/07 02:29:16 imp Exp
 #include <linux/slab.h>
 #include <linux/uio.h>
 
-#include <crypto/cryptodev.h>
+#include <opencrypto/crypto.h>
+#include <opencrypto/cryptodev.h>
 
 void
 cuio_copydata(struct uio* uio, int off, int len, caddr_t cp)
