@@ -46,7 +46,8 @@
 #include <linux/uio.h>
 #include <asm/scatterlist.h>
 
-#include <crypto/cryptodev.h>
+#include <opencrypto/crypto.h>
+#include <opencrypto/cryptodev.h>
 
 #include <IxTypes.h>
 #include <IxOsBuffMgt.h>
