@@ -43,7 +43,8 @@
 #include <linux/spinlock.h>
 #include <linux/version.h>
 #include <linux/interrupt.h>
-#include <crypto/cryptodev.h>
+#include <opencrypto/crypto.h>
+#include <opencrypto/cryptodev.h>
 
 #ifdef I_HAVE_AN_XSCALE_WITH_INTEL_SDK
 #define BENCH_IXP_ACCESS_LIB 1
