@@ -64,10 +64,12 @@ __FBSDID("$FreeBSD: src/sys/dev/safe/safe.c,v 1.8 2005/03/01 08:58:04 imp Exp $"
  * SafeNet SafeXcel-1141 hardware crypto accelerator
  */
 
-#include <crypto/cryptodev.h>
+#include <opencrypto/cryptodev.h>
+#include <opencrypto/crypto.h>
+
 #include <linux/uio.h>
-#include <safe/safereg.h>
-#include <safe/safevar.h>
+#include "safereg.h"
+#include "safevar.h"
 
 #define read_random(p,l) get_random_bytes(p,l)
 
@@ -112,10 +114,10 @@ __FBSDID("$FreeBSD: src/sys/dev/safe/safe.c,v 1.8 2005/03/01 08:58:04 imp Exp $"
 #ifdef __BIG_ENDIAN
 #define BYTE_ORDER BIG_ENDIAN
 #endif
-#include <safe/md5.h>
-#include <safe/md5.c>
-#include <safe/sha1.h>
-#include <safe/sha1.c>
+#include "md5.h"
+#include "md5.c"
+#include "sha1.h"
+#include "sha1.c"
 
 u_int8_t hmac_ipad_buffer[64] = {
     0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
@@ -144,15 +146,15 @@ u_int8_t hmac_opad_buffer[64] = {
 struct safe_stats safestats;
 
 static	int debug = 0;
-MODULE_PARM(debug, "i");
+module_param(debug, int, 0644);
 MODULE_PARM_DESC(debug, "Enable debug");
 
 #ifndef SAFE_NO_RNG
 static	int safe_rngbufsize = 8;		/* 32 bytes each read  */
-MODULE_PARM(safe_rngbufsize, "i");
+module_param(safe_rngbufsize, int, 0644);
 MODULE_PARM_DESC(safe_rngbufsize, "RNG polling buffer size (32-bit words)");
 static	int safe_rngmaxalarm = 8;		/* max alarms before reset */
-MODULE_PARM(safe_rngmaxalarm, "i");
+module_param(safe_rngmaxalarm, int, 0644);
 MODULE_PARM_DESC(safe_rngmaxalarm, "RNG max alarms before reset");
 #endif /* SAFE_NO_RNG */
 
@@ -540,7 +542,7 @@ safe_newsession(void *arg, u_int32_t *sidp, struct cryptoini *cri)
 			MD5Update(&md5ctx, macini->cri_key,
 			    macini->cri_klen / 8);
 			MD5Update(&md5ctx, hmac_ipad_buffer,
-			    HMAC_BLOCK_LEN - (macini->cri_klen / 8));
+			    MD5_HMAC_BLOCK_LEN - (macini->cri_klen / 8));
 			bcopy(md5ctx.md5_st8, ses->ses_hminner,
 			    sizeof(md5ctx.md5_st8));
 		} else {
@@ -548,7 +550,7 @@ safe_newsession(void *arg, u_int32_t *sidp, struct cryptoini *cri)
 			SHA1Update(&sha1ctx, macini->cri_key,
 			    macini->cri_klen / 8);
 			SHA1Update(&sha1ctx, hmac_ipad_buffer,
-			    HMAC_BLOCK_LEN - (macini->cri_klen / 8));
+			    SHA1_HMAC_BLOCK_LEN - (macini->cri_klen / 8));
 			bcopy(sha1ctx.h.b32, ses->ses_hminner,
 			    sizeof(sha1ctx.h.b32));
 		}
@@ -561,7 +563,7 @@ safe_newsession(void *arg, u_int32_t *sidp, struct cryptoini *cri)
 			MD5Update(&md5ctx, macini->cri_key,
 			    macini->cri_klen / 8);
 			MD5Update(&md5ctx, hmac_opad_buffer,
-			    HMAC_BLOCK_LEN - (macini->cri_klen / 8));
+			    MD5_HMAC_BLOCK_LEN - (macini->cri_klen / 8));
 			bcopy(md5ctx.md5_st8, ses->ses_hmouter,
 			    sizeof(md5ctx.md5_st8));
 		} else {
@@ -569,7 +571,7 @@ safe_newsession(void *arg, u_int32_t *sidp, struct cryptoini *cri)
 			SHA1Update(&sha1ctx, macini->cri_key,
 			    macini->cri_klen / 8);
 			SHA1Update(&sha1ctx, hmac_opad_buffer,
-			    HMAC_BLOCK_LEN - (macini->cri_klen / 8));
+			    SHA1_HMAC_BLOCK_LEN - (macini->cri_klen / 8));
 			bcopy(sha1ctx.h.b32, ses->ses_hmouter,
 			    sizeof(sha1ctx.h.b32));
 		}
