@@ -126,7 +126,8 @@
 #include <linux/platform_device.h>
 #endif
 
-#include <crypto/cryptodev.h>
+#include <opencrypto/crypto.h>
+#include <opencrypto/cryptodev.h>
 
 #define DRV_NAME "talitos" 
 
@@ -160,7 +161,7 @@ static int talitos_probe(struct platform_device *pdev);
 static int talitos_remove(struct platform_device *pdev);
 
 static int debug = 0;
-MODULE_PARM(debug, "i");
+module_param(debug, int, 0644);
 MODULE_PARM_DESC(debug, "Enable debug");
 
 static inline void talitos_write(volatile unsigned *addr, u32 val)
