@@ -331,8 +331,9 @@ ipsec_spi_get_info(char *buffer,
 			}
 			
 			if(sa_p->ocf_in_use) {
-				len += ipsec_snprintf(buffer+len, length-len, " cryptoid=%d",
-					       sa_p->ocf_cryptoid);
+				len += ipsec_snprintf(buffer+len, length-len, " cryptoid=%u/%u",
+						      (sa_p->ocf_cryptoid>>32),
+						      ( sa_p->ocf_cryptoid&0xffffffff));
 			}
 			
 			len += ipsec_snprintf(buffer+len, length-len, " life(c,s,h)=");
