@@ -2469,6 +2469,139 @@ static void update_retransmit_history(struct state *st, struct msg_digest *md)
 	}
 }	
 
+static void fmt_ipsec_sa_established(struct state *st, char *sadetails, int sad_len)
+{
+    char *b = sadetails;
+    const char *ini = " {";
+    const char *fin = "";
+    
+    strcpy(sadetails,
+	   (st->st_connection->policy & POLICY_TUNNEL ?
+	    " tunnel mode" : " transport mode"));
+    b += strlen(sadetails);
+    
+    /* -1 is to leave space for "fin" */
+    
+    if(st->st_esp.present)
+    {
+	const char *natinfo="";
+	
+	if((st->st_connection->spd.that.host_port != IKE_UDP_PORT
+	    && st->st_connection->spd.that.host_port != 0)
+	   || st->st_connection->forceencaps) {
+	    natinfo="/NAT";
+	}
+	snprintf(b, sad_len-(b-sadetails)-1
+		 , "%sESP%s=>0x%08lx <0x%08lx xfrm=%s_%d-%s"
+		 , ini
+		 , natinfo
+		 , (unsigned long)ntohl(st->st_esp.attrs.spi)
+		 , (unsigned long)ntohl(st->st_esp.our_spi)
+		 , enum_show(&esp_transformid_names, st->st_esp.attrs.transid)+strlen("ESP_")
+		 , st->st_esp.attrs.key_len
+		 , enum_show(&auth_alg_names, st->st_esp.attrs.auth)+strlen("AUTH_ALGORITHM_"));
+	ini = " ";
+	fin = "}";
+    }
+    /* advance b to end of string */
+    b = b + strlen(b);
+    
+    if(st->st_ah.present)
+    {
+	snprintf(b, sad_len-(b-sadetails)-1
+		 , "%sAH=>0x%08lx <0x%08lx"
+		 , ini
+		 , (unsigned long)ntohl(st->st_ah.attrs.spi)
+		 , (unsigned long)ntohl(st->st_ah.our_spi));
+	ini = " ";
+	fin = "}";
+    }
+    /* advance b to end of string */
+    b = b + strlen(b);
+    
+    if(st->st_ipcomp.present)
+    {
+	snprintf(b, sad_len-(b-sadetails)-1
+		 , "%sIPCOMP(%s)=>0x%08lx <0x%08lx"
+		 , ini
+		 , enum_name(&ipcomp_transformid_names, st->st_ipcomp.attrs.transid)+sizeof("IPCOMP")
+		 , (unsigned long)ntohl(st->st_ipcomp.attrs.spi)
+		 , (unsigned long)ntohl(st->st_ipcomp.our_spi));
+	ini = " ";
+	fin = "}";
+    }
+    
+    /* advance b to end of string */
+    b = b + strlen(b);
+#ifdef NAT_TRAVERSAL		    
+    {
+	char oa[ADDRTOT_BUF];
+	
+	strcpy(oa, "none");
+	if(!isanyaddr(&st->hidden_variables.st_nat_oa)) {
+	    addrtot(&st->hidden_variables.st_nat_oa, 0
+		    , oa, sizeof(oa));
+	}
+	snprintf(b, sad_len-(b-sadetails)-1
+		 , "%sNATOA=%s"
+		 , ini, oa);
+	ini = " ";
+	fin = "}";
+    }
+    
+    {
+	char oa[ADDRTOT_BUF+sizeof(":00000")];
+	
+	strcpy(oa, "none");
+	if(!isanyaddr(&st->hidden_variables.st_natd)) {
+	    char oa2[ADDRTOT_BUF];
+	    addrtot(&st->hidden_variables.st_natd, 0
+		    , oa2, sizeof(oa2));
+	    snprintf(oa, sizeof(oa)
+		     , "%s:%d", oa2, st->st_remoteport);
+	}
+	snprintf(b, sad_len-(b-sadetails)-1
+		 , "%sNATD=%s"
+		 , ini, oa);
+	ini = " ";
+	fin = "}";
+    }
+#endif
+    
+    /* advance b to end of string */
+    b = b + strlen(b);
+    
+    snprintf(b, sad_len-(b-sadetails)-1
+	     , "%sDPD=%s"
+	     , ini
+	     , st->hidden_variables.st_dpd_local ?
+	     "enabled" : "none");
+    
+    ini = " ";
+    fin = "}";
+    
+    strcat(b, fin);
+}
+
+static void fmt_isakmp_sa_established(struct state *st, char *sadetails, int sad_len)
+{
+
+    /* document ISAKMP SA details for admin's pleasure */
+    char *b = sadetails;
+    
+    passert(st->st_oakley.encrypter != NULL);
+    passert(st->st_oakley.hasher != NULL);
+    passert(st->st_oakley.group != NULL);
+    
+    snprintf(b, sad_len-(b-sadetails)-1
+	     , " {auth=%s cipher=%s_%d prf=%s group=modp%d}"
+	     , enum_show(&oakley_auth_names, st->st_oakley.auth)
+	     , st->st_oakley.encrypter->common.name
+	     , st->st_oakley.enckeylen
+	     , st->st_oakley.hasher->common.name
+	     , (int)st->st_oakley.group->bytes*8);
+    st->hidden_variables.st_logged_p1algos = TRUE;
+}
 
 /* complete job started by the state-specific state transition function */
 
@@ -2715,132 +2848,11 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		/* document IPsec SA details for admin's pleasure */
 		if(IS_IPSEC_SA_ESTABLISHED(st->st_state))
 		{
-		    char *b = sadetails;
-		    const char *ini = " {";
-		    const char *fin = "";
+		    fmt_ipsec_sa_established(st, sadetails, sizeof(sadetails));
 
-		    strcpy(sadetails,
-			   (st->st_connection->policy & POLICY_TUNNEL ?
-			    " tunnel mode" : " transport mode"));
-		    b += strlen(sadetails);
-
-		    /* -1 is to leave space for "fin" */
-
-		    if(st->st_esp.present)
-		    {
-			const char *natinfo="";
-
-			if((st->st_connection->spd.that.host_port != IKE_UDP_PORT
-			    && st->st_connection->spd.that.host_port != 0)
-			   || st->st_connection->forceencaps) {
-			    natinfo="/NAT";
-			}
-			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-				 , "%sESP%s=>0x%08lx <0x%08lx xfrm=%s_%d-%s"
-				 , ini
-				 , natinfo
-				 , (unsigned long)ntohl(st->st_esp.attrs.spi)
-				 , (unsigned long)ntohl(st->st_esp.our_spi)
-				 , enum_show(&esp_transformid_names, st->st_esp.attrs.transid)+strlen("ESP_")
-				 , st->st_esp.attrs.key_len
-				 , enum_show(&auth_alg_names, st->st_esp.attrs.auth)+strlen("AUTH_ALGORITHM_"));
-			ini = " ";
-			fin = "}";
-		    }
-		    /* advance b to end of string */
-		    b = b + strlen(b);
-		    
-		    if(st->st_ah.present)
-		    {
-			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-				 , "%sAH=>0x%08lx <0x%08lx"
-				 , ini
-				 , (unsigned long)ntohl(st->st_ah.attrs.spi)
-				 , (unsigned long)ntohl(st->st_ah.our_spi));
-			ini = " ";
-			fin = "}";
-		    }
-		    /* advance b to end of string */
-		    b = b + strlen(b);
-		    
-		    if(st->st_ipcomp.present)
-		    {
-			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-				 , "%sIPCOMP=>0x%08lx <0x%08lx"
-				 , ini
-				 , (unsigned long)ntohl(st->st_ipcomp.attrs.spi)
-				 , (unsigned long)ntohl(st->st_ipcomp.our_spi));
-			ini = " ";
-			fin = "}";
-		    }
-
-		    /* advance b to end of string */
-		    b = b + strlen(b);
-#ifdef NAT_TRAVERSAL		    
-		    {
-			char oa[ADDRTOT_BUF];
-
-			strcpy(oa, "none");
-			if(!isanyaddr(&st->hidden_variables.st_nat_oa)) {
-			  addrtot(&st->hidden_variables.st_nat_oa, 0
-				  , oa, sizeof(oa));
-			}
-			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-				 , "%sNATOA=%s"
-				 , ini, oa);
-			ini = " ";
-			fin = "}";
-		    }
-
-		    {
-			char oa[ADDRTOT_BUF+sizeof(":00000")];
-
-			strcpy(oa, "none");
-			if(!isanyaddr(&st->hidden_variables.st_natd)) {
-			    char oa2[ADDRTOT_BUF];
-			    addrtot(&st->hidden_variables.st_natd, 0
-				    , oa2, sizeof(oa2));
-			    snprintf(oa, sizeof(oa)
-				     , "%s:%d", oa2, st->st_remoteport);
-			}
-			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-				 , "%sNATD=%s"
-				 , ini, oa);
-			ini = " ";
-			fin = "}";
-		    }
-#endif
-
-		    /* advance b to end of string */
-		    b = b + strlen(b);
-		    
-		    snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-			     , "%sDPD=%s"
-			     , ini
-			     , st->hidden_variables.st_dpd_local ?
-			     "enabled" : "none");
-
-		    ini = " ";
-		    fin = "}";
-
-		    strcat(b, fin);
 		} else if(IS_ISAKMP_SA_ESTABLISHED(st->st_state)
-			  && !st->hidden_variables.st_logged_p1algos) {
-		    /* document ISAKMP SA details for admin's pleasure */
-		    char *b = sadetails;
-
-		    passert(st->st_oakley.encrypter != NULL);
-		    passert(st->st_oakley.hasher != NULL);
-		    passert(st->st_oakley.group != NULL);
-
-		    snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-			     , " {auth=%s cipher=%s_%d prf=%s group=modp%d}"
-			     , enum_show(&oakley_auth_names, st->st_oakley.auth)
-			     , st->st_oakley.encrypter->common.name
-			     , st->st_oakley.enckeylen
-			     , st->st_oakley.hasher->common.name
-			     , (int)st->st_oakley.group->bytes*8);
-		    st->hidden_variables.st_logged_p1algos = TRUE;
+		      && !st->hidden_variables.st_logged_p1algos) {
+		    fmt_isakmp_sa_established(st, sadetails,sizeof(sadetails));
 		}
 
 		if (IS_ISAKMP_SA_ESTABLISHED(st->st_state)
@@ -2850,7 +2862,7 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		    w = RC_SUCCESS;
 		}
 
-		/* tell whack and logs our progress */
+                /* tell whack and logs our progress */
 		loglog(w
 		       , "%s: %s%s"
 		       , enum_name(&state_names, st->st_state)
