@@ -298,6 +298,9 @@ aggr_inI1_outR1_common(struct msg_digest *md
 
     insert_state(st);	/* needs cookies, connection, and msgid (0) */
 
+    /* copy the quirks we might have accumulated */
+    copy_quirks(&st->quirks,&md->quirks);
+
     st->st_doi = ISAKMP_DOI_IPSEC;
     st->st_situation = SIT_IDENTITY_ONLY; /* We only support this */
 
@@ -955,12 +958,16 @@ aggr_outI1(int whack_sock,
 	ke->md->st = st;
 	set_suspended(st, ke->md);
 
-	passert(st->st_sec_in_use == FALSE);
-	ke->ke_pcrc.pcrc_func = aggr_outI1_continue;
-	e = build_ke(&ke->ke_pcrc, st, st->st_oakley.group, importance);
-	if(e != STF_SUSPEND) {
-	    loglog(RC_CRYPTOFAILED, "system too busy");
-	    delete_state(st);
+	if (!st->st_sec_in_use) {
+	    ke->ke_pcrc.pcrc_func = aggr_outI1_continue;
+	    e = build_ke(&ke->ke_pcrc, st, st->st_oakley.group, importance);
+	    if(e != STF_SUSPEND && e != STF_INLINE) {
+	      loglog(RC_CRYPTOFAILED, "system too busy");
+	      delete_state(st);
+	    }
+	} else {
+	    e = aggr_outI1_tail((struct pluto_crypto_req_cont *)ke
+					, NULL);
 	}
 
 	reset_globals();
