@@ -31,7 +31,8 @@
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/types.h>
-#if defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
+#if !defined(linux)
+#include <sys/param.h>
 #include <sys/sysctl.h>
 #endif
 
@@ -179,7 +180,8 @@ helper_passert_fail(const char *pred_str
 
 void pluto_crypto_helper(int fd, int helpernum)
 {
-    FILE *io = fdopen(fd, "ab+");
+    FILE *in  = fdopen(fd, "rb");
+    FILE *out = fdopen(fd, "wb");
     long reqbuf[PCR_REQ_SIZE/sizeof(long)];
     struct pluto_crypto_req *r;
 
@@ -191,10 +193,10 @@ void pluto_crypto_helper(int fd, int helpernum)
     setpriority(PRIO_PROCESS, 0, 10);
 
     DBG(DBG_CONTROL, DBG_log("helper %d waiting on fd: %d"
-			     , helpernum, fileno(io)));
+			     , helpernum, fileno(in)));
 
     memset(reqbuf, 0, sizeof(reqbuf));
-    while(fread((char*)reqbuf, sizeof(r->pcr_len), 1, io) == 1) {
+    while(fread((char*)reqbuf, sizeof(r->pcr_len), 1, in) == 1) {
 	int restlen;
 	int actnum;
 	unsigned char *reqrest = ((unsigned char *)reqbuf)+sizeof(r->pcr_len);
@@ -203,10 +205,15 @@ void pluto_crypto_helper(int fd, int helpernum)
 	restlen = r->pcr_len-sizeof(r->pcr_len);
 	
 	passert(restlen < (signed)PCR_REQ_SIZE);
+	passert(restlen > 0);
 
-	actnum = fread(reqrest, restlen, 1, io);
+	actnum = fread(reqrest, 1, restlen, in);
 	/* okay, got a basic size, read the rest of it */
-	if(actnum != 1) {
+
+	DBG(DBG_CONTROL, DBG_log("helper %d read %d+4/%d bytesfd: %d"
+				 , helpernum, actnum, r->pcr_len, fileno(in)));
+
+	if(actnum != restlen) {
 	    /* faulty read. die, parent will restart us */
 	    loglog(RC_LOG_SERIOUS, "cryptographic helper(%d) fread(%d)=%d failed: %s\n",
 		   getpid(), restlen, actnum, strerror(errno));
@@ -223,7 +230,8 @@ void pluto_crypto_helper(int fd, int helpernum)
 
 	pluto_do_crypto_op(r);
 
-	actnum = fwrite((unsigned char *)r, r->pcr_len, 1, io);
+	actnum = fwrite((unsigned char *)r, r->pcr_len, 1, out);
+	fflush(out);
 
 	if(actnum != 1) {
 	    loglog(RC_LOG_SERIOUS, "failed to write answer: %d", actnum);
@@ -232,16 +240,50 @@ void pluto_crypto_helper(int fd, int helpernum)
 	memset(reqbuf, 0, sizeof(reqbuf));
     }
 
-    if(!feof(io)) {
-	loglog(RC_LOG_SERIOUS, "helper %d got error: %s", helpernum, strerror(ferror(io)));
+    if(!feof(in)) {
+	loglog(RC_LOG_SERIOUS, "helper %d got error: %s", helpernum, strerror(ferror(in)));
     }
 
     /* probably normal EOF */
-    fclose(io);
+    fclose(in);
+    fclose(out);
     exit(0);
 }
 
 
+/* send the request, make sure it all goes down. */
+static bool crypto_write_request(struct pluto_crypto_worker *w
+				 ,struct pluto_crypto_req *r)
+{
+    unsigned char *wdat = (unsigned char *)r;
+    int wlen = r->pcr_len;
+    int cnt;
+    
+    DBG(DBG_CONTROL
+	, DBG_log("asking helper %d to do %s op on seq: %u (len=%u, pcw_work=%d)"
+		  , w->pcw_helpernum
+		  , enum_show(&pluto_cryptoop_names, r->pcr_type)
+		  , r->pcr_id, r->pcr_len, w->pcw_work+1));
+
+    do {
+	cnt = write(w->pcw_pipe, wdat, wlen);
+	
+	if(cnt <= 0) {
+	    return FALSE;
+	}
+	if(cnt != wlen) {
+	    DBG_log("crypto helper write failed to write all (%d<%d), retrying.\n", cnt, wlen);
+	}
+
+	wlen -= cnt;
+	w    += cnt;
+
+    } while(wlen > 0);
+
+    return TRUE;
+}
+
+
 /*
  * this function is called with a request to do some cryptographic operations
  * along with a continuation structure, which will be used to deal with
@@ -290,8 +332,9 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
     w = &pc_workers[pc_worker_num];
 
     DBG(DBG_CONTROL
-	, DBG_log("%d: w->pcw_dead: %d w->pcw_work: %d cnt: %d",
-		  pc_worker_num, w->pcw_dead, w->pcw_work, cnt));
+	, DBG_log("%d: w->pcw_dead: %d w->pcw_work: %d cnt: %d %s"
+		  , pc_worker_num, w->pcw_dead, w->pcw_work, cnt
+		  , enum_name(&pluto_cryptoimportance_names,r->pcr_pcim)));
     
     while((w->pcw_dead || (w->pcw_work >= w->pcw_maxbasicwork))
 	  && --cnt > 0) {
@@ -370,18 +413,10 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
     passert(w->pcw_pipe != -1);
     passert(w->pcw_work < w->pcw_maxcritwork);
     
-    DBG(DBG_CONTROL
-	, DBG_log("asking helper %d to do %s op on seq: %u"
-		  , w->pcw_helpernum
-		  , enum_show(&pluto_cryptoop_names, r->pcr_type)
-		  , r->pcr_id));
-
-    /* send the request, and then mark the work as having more work */
-    cnt = write(w->pcw_pipe, r, r->pcr_len);
-    if(cnt == -1) {
+    if(!crypto_write_request(w, r)) {
 	return "failed to write";
-    } 
-
+    }
+	
     w->pcw_work++;
     *toomuch = FALSE;
     return NULL;
@@ -396,8 +431,6 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
     struct pluto_crypto_req_cont *cn;
 
     if(backlogqueue_len > 0) {
-	int cnt;
-
 	passert(backlogqueue != NULL);
 	
 	cn = backlogqueue;
@@ -410,7 +443,7 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
 	    , DBG_log("removing backlog item (%d) from queue: %d left"
 		      , r->pcr_id, backlogqueue_len));
 
-	/* w points to a work. Make sure it is live */
+	/* w points to a worker. Make sure it is live */
 	if(w->pcw_pid == -1) {
 	    init_crypto_helper(w, pc_worker_num);
 	    if(w->pcw_pid == -1) {
@@ -430,15 +463,8 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
 	passert(w->pcw_pipe != -1);
 	passert(w->pcw_work > 0);
     
-	DBG(DBG_CONTROL
-	    , DBG_log("asking helper %d to do %s op on seq: %u"
-		      , w->pcw_helpernum
-		      , enum_show(&pluto_cryptoop_names, r->pcr_type)
-		      , r->pcr_id));
-	
-	/* send the request, and then mark the work as having more work */
-	cnt = write(w->pcw_pipe, r, r->pcr_len);
-	if(cnt == -1) {
+	/* send the request, and then mark the worker as having more work */
+	if(!crypto_write_request(w, r)) {
 	    /* XXX invoke callback with failure */
 	    passert(0);
 	    return;
@@ -525,10 +551,7 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     int actlen;
     struct pluto_crypto_req_cont *cn, **cnp;
 
-    /* we can accept more work now that we are about to read from the pipe */
-    w->pcw_work--;
-
-    DBG(DBG_CRYPT, DBG_log("helper %u has work (cnt now %d)"
+    DBG(DBG_CRYPT, DBG_log("helper %u has finished work (cnt now %d)"
 			   ,w->pcw_helpernum
 			   ,w->pcw_work));
 
@@ -550,6 +573,9 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
 	return;
     }
 
+    /* we can accept more work now that we have read from the pipe */
+    w->pcw_work--;
+
     r = (struct pluto_crypto_req *)reqbuf;
 
     if(r->pcr_len > sizeof(reqbuf)) {
@@ -833,7 +859,7 @@ void init_crypto_helpers(int nhelpers)
     /* if nhelpers == 0, then we do all the work ourselves */
     if(nhelpers == -1) {
 	int ncpu_online;
-#if !(defined(macintosh) || (defined(__MACH__) && defined(__APPLE__)))
+#if defined(_SC_NPROCESSORS_ONLN)
       ncpu_online = sysconf(_SC_NPROCESSORS_ONLN);
 #else
       int mib[2], numcpu;
