@@ -168,7 +168,7 @@ main(int argc, char *argv[])
     cfg = confread_load(configfile, &err, NULL);
     
     if(!cfg) {
-	printf("config file: %s can not be loaded\n", configfile);
+	printf("config file: %s can not be loaded: %s\n", configfile, err);
 	exit(3);
     }
     
