@@ -125,14 +125,6 @@ consolediff() {
 		*.awk) cleanups="$cleanups | awk -f $FIXUPDIR/$fixup";;
 		    *) echo Unknown fixup type: $fixup;;
             esac
-	elif [ -f $FIXUPDIR2/$fixup ]
-	then
-	    case $fixup in
-		*.sed) cleanups="$cleanups | sed -f $FIXUPDIR2/$fixup";;
-		*.pl)  cleanups="$cleanups | perl $FIXUPDIR2/$fixup";;
-		*.awk) cleanups="$cleanups | awk -f $FIXUPDIR2/$fixup";;
-		    *) echo Unknown fixup type: $fixup;;
-            esac
 	else
 	    echo Fixup $fixup not found.
 	    success="missing fixup"
@@ -1752,25 +1744,23 @@ buildtest() {
 #          OBJDIRTOP=  set to location of object files
 # 
 #
-# testparams.sh should set PROGRAMS= to a list of subdirs of programs/
-#                that must be built before using the test. This allows
-#                additional modules to be built.
+# testparams.sh should set PROGRAMS= to a list of subdirs of programs/ that must
+#                be built before using the test. This allows additional modules
+#                to be built.
 #
-# If there is a Makefile in the subdir, it will be invoked as
-# "make checkprograms". It will have the above variables as well,
-# and make get the build environment with 
-#    include ${ROOTDIR}/programs/Makefile.program
+# If there is a Makefile in the subdir, it will be invoked as "make checkprograms"
 #
-# The stdout of the script will be set to an output file, which will then
-# be sanitized using the normal set of fixup scripts.
+# The stdout of the script will be set to an output file, which will then be sanitized
+# using the normal set of fixup scripts.
 #          
 #
 ###################################
 
 do_unittest() {
 
+    success=true
     export ROOTDIR=${OPENSWANSRCDIR}
-    eval `(cd $ROOTDIR; make env)`
+    eval `(cd $ROOTDIR; ${MAKE} env)`
     failnum=1
 
     if [ ! -x "$TESTSCRIPT" ]; then echo "TESTSCRIPT=$TESTSCRIPT is not executable"; exit 41; fi
@@ -1779,11 +1769,11 @@ do_unittest() {
     (cd ${ROOTDIR}/programs;
      for program in ${PROGRAMS}
      do
-	if [ -d $program ]; then (cd $program && make programs checkprograms ); fi
+	if [ -d $program ]; then (cd $program && ${MAKE} programs checkprograms ); fi
      done)
 
     # if there is a makefile, run it and bail if fails
-    [ -f Makefile ] && make checkprograms
+    [ -f Makefile ] && ${MAKE} checkprograms
 
     # make sure we get all core dumps!
     ulimit -c unlimited
@@ -1817,7 +1807,7 @@ unittest() {
 
     echo '**** make unittest RUNNING '$testcase' ****'
 
-    echo Running $testobj
+    echo Running $testcase
     ( preptest $testcase unittest false && do_unittest )
     stat=$?
 
